@@ -407,6 +407,9 @@ coef.of.var <- function(x, pop = TRUE) {
 mean.abs.dev <- function(x) {
     mean_fast(abs(x - mean_fast(x, TRUE)), TRUE)
 }
+rms <- function(x) {
+    sqrt(mean_fast(x^2))
+}
 geom.mean <- function(y) {
     exp(mean_fast(log(y[is.finite(log(y))]), TRUE))
 }
@@ -686,6 +689,96 @@ process.s.weights <- function(s.weights, data = NULL) {
     else s.weights <- NULL
     return(s.weights)
 }
+process.focal.and.estimand <- function(focal, estimand, treat, treat.type, treated = NULL) {
+    reported.estimand <- estimand
+    
+    if (!has.treat.type(treat)) treat <- assign.treat.type(treat)
+    treat.type <- get.treat.type(treat)
+    
+    #Check focal
+    if (treat.type == "multinomial") {
+        unique.treat <- unique(treat, nmax = length(treat)/4)
+        if (estimand %nin% c("ATT", "ATC") && is_not_null(focal)) {
+            warning(paste(estimand, "is not compatible with 'focal'. Setting 'estimand' to \"ATT\"."), call. = FALSE)
+            reported.estimand <- estimand <- "ATT"
+        }
+        
+        if (estimand == "ATT") {
+            if (is_null(focal)) {
+                if (is_null(treated) || treated %nin% unique.treat) {
+                    stop("When estimand = \"ATT\" for multinomial treatments, an argument must be supplied to 'focal'.", call. = FALSE)
+                }
+                focal <- treated
+            }
+        }
+        else if (estimand == "ATC") {
+            if (is_null(focal)) {
+                stop("When estimand = \"ATC\" for multinomial treatments, an argument must be supplied to 'focal'.", call. = FALSE)
+            }
+        }
+    }
+    else if (treat.type == "binary") {
+        unique.treat <- unique(treat, nmax = 2)
+        unique.treat.bin <- unique(binarize(treat), nmax = 2)
+        if (estimand %nin% c("ATT", "ATC") && is_not_null(focal)) {
+            warning(paste(estimand, "is not compatible with 'focal'. Setting 'estimand' to \"ATT\"."), call. = FALSE)
+            reported.estimand <- estimand <- "ATT"
+        }
+        
+        if (estimand == "ATT") {
+            if (is_null(focal)) {
+                if (is_null(treated) || treated %nin% unique.treat) {
+                    if (all(as.character(unique.treat.bin) == as.character(unique.treat))) {
+                        #If is 0/1
+                        treated <- unique.treat[unique.treat.bin == 1]
+                    }
+                    else {
+                        treated <- names(which.min(table(treat))) #Smaller group is treated
+                        message(paste0("Assuming ", word_list(treated, quotes = !is.numeric(treat), is.are = TRUE),
+                                       " the treated level. If not, supply an argument to 'focal'."))
+                    }
+                }
+                focal <- treated
+            }
+            else {
+                if (is_null(treated) || treated %nin% unique.treat) {
+                    treated <- focal
+                }
+            }
+        }
+        else if (estimand == "ATC") {
+            if (is_null(focal)) {
+                if (is_null(treated) || treated %nin% unique.treat) {
+                    
+                    if (all(as.character(unique.treat.bin) == as.character(unique.treat))) {
+                        treated <- unique.treat[unique.treat.bin == 1]
+                    }
+                    else {
+                        treated <- names(which.min(table(treat))) #Smaller group is treated
+                        message(paste0("Assuming ", word_list(unique.treat[unique.treat %nin% treated], quotes = !is.numeric(treat), is.are = TRUE),
+                                       " the control level. If not, supply an argument to 'focal'."))
+                    }
+                }
+                focal <- unique.treat[unique.treat %nin% treated]
+            }
+            else {
+                if (is_null(treated) || treated %nin% unique.treat) {
+                    treated <- unique.treat[unique.treat %nin% focal]
+                }
+            }
+            estimand <- "ATT"
+        }
+    }
+    
+    if (is_not_null(focal) && (length(focal) > 1L || focal %nin% unique.treat)) {
+        stop("The argument supplied to 'focal' must be the name of a level of treatment.", call. = FALSE)
+    }
+    
+    return(list(focal = as.character(focal),
+                estimand = estimand,
+                reported.estimand = reported.estimand,
+                treated = if (is.factor(treated)) as.character(treated) else treated))
+}
 
 #Uniqueness
 nunique <- function(x, nmax = NA, na.rm = TRUE) {
@@ -732,6 +825,7 @@ make_list <- function(n) {
     else if (is_(n, "atomic")) {
         setNames(vector("list", length(n)), as.character(n))
     }
+    else stop("'n' must be an integer(ish) scalar or an atomic variable.")
 }
 make_df <- function(ncol, nrow = 0, types = "numeric") {
     if (length(ncol) == 1L && is.numeric(ncol)) {
@@ -768,7 +862,7 @@ ifelse_ <- function(...) {
     if (dotlen %% 2 == 0) stop("ifelse_ must have an odd number of arguments: pairs of test/yes, and one no.")
     out <- ...elt(dotlen)
     if (dotlen > 1) {
-        if (!is_(out, "atomic")) stop("The last entry to ifelse_ must be atomic or factor.")
+        if (!is_(out, "atomic")) stop("The last entry to ifelse_ must be atomic.")
         if (length(out) == 1) out <- rep(out, length(..1))
         n <- length(out)
         for (i in seq_len((dotlen - 1)/2)) {
@@ -777,13 +871,13 @@ ifelse_ <- function(...) {
             if (length(yes) == 1) yes <- rep(yes, n)
             if (length(yes) != n || length(test) != n) stop("All entries must have the same length.")
             if (!is.logical(test)) stop(paste("The", ordinal(2*i - 1), "entry to ifelse_ must be logical."))
-            if (!is_(yes, "atomic")) stop(paste("The", ordinal(2*i), "entry to ifelse_ must be atomic or factor."))
+            if (!is_(yes, "atomic")) stop(paste("The", ordinal(2*i), "entry to ifelse_ must be atomic."))
             pos <- which(test)
             out[pos] <- yes[pos]
         }
     }
     else {
-        if (!is_(out, "atomic")) stop("The first entry to ifelse_ must be atomic or factor.")
+        if (!is_(out, "atomic")) stop("The first entry to ifelse_ must be atomic.")
     }
     return(out)
 }
@@ -944,4 +1038,5 @@ is.formula <- function(f, sides = NULL) {
         res <- res && length(f) == sides + 1
     }
     return(res)
-}
\ No newline at end of file
+}
+if (getRversion() < 3.6) str2expression <- function(text) parse(text=text, keep.source=FALSE)
\ No newline at end of file
